package solutions.bitbadger.documents.core.tests.integration

import solutions.bitbadger.documents.*
import solutions.bitbadger.documents.core.tests.*
import solutions.bitbadger.documents.java.extensions.*
import solutions.bitbadger.documents.java.Results
import solutions.bitbadger.documents.query.CountQuery
import solutions.bitbadger.documents.query.DeleteQuery
import solutions.bitbadger.documents.query.FindQuery
import kotlin.test.*

/**
 * Integration tests for the `Custom` object
 */
object CustomFunctions {

    fun listEmpty(db: ThrowawayDatabase) {
        JsonDocument.load(db)
        db.conn.deleteByFields(TEST_TABLE, listOf(Field.exists(Configuration.idField)))
        val result =
            db.conn.customList(FindQuery.all(TEST_TABLE), listOf(), JsonDocument::class.java, Results::fromData)
        assertEquals(0, result.size, "There should have been no results")
    }

    fun listAll(db: ThrowawayDatabase) {
        JsonDocument.load(db)
        val result =
            db.conn.customList(FindQuery.all(TEST_TABLE), listOf(), JsonDocument::class.java, Results::fromData)
        assertEquals(5, result.size, "There should have been 5 results")
    }

    fun singleNone(db: ThrowawayDatabase) =
        assertFalse(
            db.conn.customSingle(
                FindQuery.all(TEST_TABLE),
                listOf(),
                JsonDocument::class.java,
                Results::fromData
            ).isPresent,
            "There should not have been a document returned"
        )

    fun singleOne(db: ThrowawayDatabase) {
        JsonDocument.load(db)
        assertTrue(
            db.conn.customSingle(
                FindQuery.all(TEST_TABLE),
                listOf(),
                JsonDocument::class.java,
                Results::fromData
            ).isPresent,
            "There should not have been a document returned"
        )
    }

    fun nonQueryChanges(db: ThrowawayDatabase) {
        JsonDocument.load(db)
        assertEquals(
            5L, db.conn.customScalar(CountQuery.all(TEST_TABLE), listOf(), Long::class.java, Results::toCount),
            "There should have been 5 documents in the table"
        )
        db.conn.customNonQuery("DELETE FROM $TEST_TABLE")
        assertEquals(
            0L, db.conn.customScalar(CountQuery.all(TEST_TABLE), listOf(), Long::class.java, Results::toCount),
            "There should have been no documents in the table"
        )
    }

    fun nonQueryNoChanges(db: ThrowawayDatabase) {
        JsonDocument.load(db)
        assertEquals(
            5L, db.conn.customScalar(CountQuery.all(TEST_TABLE), listOf(), Long::class.java, Results::toCount),
            "There should have been 5 documents in the table"
        )
        db.conn.customNonQuery(
            DeleteQuery.byId(TEST_TABLE, "eighty-two"),
            listOf(Parameter(":id", ParameterType.STRING, "eighty-two"))
        )
        assertEquals(
            5L, db.conn.customScalar(CountQuery.all(TEST_TABLE), listOf(), Long::class.java, Results::toCount),
            "There should still have been 5 documents in the table"
        )
    }

    fun scalar(db: ThrowawayDatabase) {
        JsonDocument.load(db)
        assertEquals(
            3L,
            db.conn.customScalar(
                "SELECT 3 AS it FROM $TEST_TABLE LIMIT 1",
                listOf(),
                Long::class.java,
                Results::toCount
            ),
            "The number 3 should have been returned"
        )
    }
}
